#!/bin/bash

# Author: Victor Correal (victor.correal@bsc.es)
# Date  : 28/06/2024

# IMPORTS:
SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
source ${SCRIPT_DIR}/bash_utils.sh
source ${SCRIPT_DIR}/run_nemo_utils.sh
source ${SCRIPT_DIR}/run_setups.sh

# SIM VARIABLES:
NEMO=
XIOS_BIN=
XIOS_MODE=
ENV_FILE=
RUNDIR=
RESOLUTION=
FORCE_RUN=false
DRY=false
TSTEPS=DEFAULT

# SLURM VARIABLES:
ACCOUNT=DEFAULT
NODES=DEFAULT
NODES_XIOS=DEFAULT
QUEUE=DEFAULT
TIME_LIMIT=DEFAULT
MPI_PER_NODE=DEFAULT
OMP_PER_TASK=DEFAULT
SLURM_HIGHMEM=false
SLURM_WAIT=false
SLURM_CONTIGUOUS=false
SLURM_ARGS=""

# PARSE ARGS && USER CHECK(Y/N):
parse_args "$@"

check_required_args
check_dirs_and_bins
check_slurm_options
print_options
check_user

# COPY BINARIES & ENV:
mkdir -p $RUNDIR

cp -rL $NEMO/nemo $RUNDIR/nemo.exe
cp $XIOS_BIN $RUNDIR/xios_server.exe
ln -sf $PWD/$ENV_FILE $RUNDIR/env.sh

cp $NEMO/file_def_nemo-ice.xml $RUNDIR
cp $NEMO/file_def_nemo-oce.xml $RUNDIR

FLS="axis_def_nemo.xml grid_def_nemo.xml domain_def_nemo.xml field_def_nemo-ice.xml field_def_nemo-oce.xml  "
FLS+="namelist_ref namelist_ice_ref namelist_top_ref "

for f in `echo $FLS` ; do
    cp $NEMO/../../SHARED/${f} $RUNDIR
done

# COPY CONFIGURATIONS, RESTARTS & FORCINGS:
info "using $RESOLUTION, loading inputs into rundir"
case "$RESOLUTION" in
    ORCA025-FRO)
        orca025_common
    ;;
    ORCA025-FRO-IO)
        orca025_common
        common_io
    ;;
    ORCA025-RK3)
        orca025_common
        orca025_rk3_common
    ;;
    ORCA025-RK3-IO)
        orca025_common
        orca025_rk3_common
        common_io
    ;;
    ORCA025-RK3-2400)
        orca025_common
        orca025_rk3_2400
    ;;
    ORCA025-RK3-3600)
        orca025_common
        orca025_rk3_3600
    ;;
    ORCA12-FRO)
        orca12_common
    ;;
    ORCA12-RK3)
        orca12_common
    ;;
    ORCA12-FRO-IO)
        orca12_common
        common_io
    ;;
    ORCA12-RK3-IO)
        orca12_common
        common_io
    ;;
    ORCA36-RK3)
        orca36_common
        orca36_rk3
    ;;
    ORCA36-FRO)
        orca36_common
        
    ;;
    *)
        fatal "INVALID RESOLUTION: $RESOLUTION"
    ;;
esac

# MANAGE NAMELIST:
if [ "$TSTEPS" != "DEFAULT" ]; then
    change_namelist_parameter $RUNDIR/namelist_cfg nn_itend $TSTEPS
fi


# COPY SLURM SCRIPT:
info "SETTING UP SLURM SCRIPT..."
case "$XIOS_MODE" in
    no-xios)
        info "XIOS is disabled"
        cp ${SCRIPT_DIR}/run_nemo_no_xios.slurm $RUNDIR/run_nemo.slurm
    ;;
    client)
        info "XIOS is running in client mode"
        cp ${SCRIPT_DIR}/run_nemo.slurm $RUNDIR/run_nemo.slurm
    ;;
    client-memcheck)
        info "XIOS is running in client mode with memcheck"
        cp ${SCRIPT_DIR}/run_nemo_mem_check.slurm $RUNDIR/run_nemo.slurm
        cp ${SCRIPT_DIR}/free-wrapper.sh $RUNDIR
    ;;
    server)
        info "XIOS is running in server"
        cp ${SCRIPT_DIR}/run_nemo_xios_server.slurm $RUNDIR/run_nemo.slurm
    ;;
    server-memcheck)
        info "XIOS is running in server with memcheck"
        cp ${SCRIPT_DIR}/run_nemo_xios_server_mem_check.slurm $RUNDIR/run_nemo.slurm
        cp ${SCRIPT_DIR}/free-wrapper.sh $RUNDIR
    ;;
    *)
        fatal "INVALID XIOS_MODE: $XIOS_MODE"
    ;;
esac

#cp ${SCRIPT_DIR}/run_nemo_het.slurm $RUNDIR/run_nemo.slurm


# SUBMIT IF ! DRY:
if [ $DRY = true ]; then
    success "DRY -> NO SUBMIT!"
else
    cd $RUNDIR
    JOB_ID=$(sbatch $SLURM_ARGS --parsable run_nemo.slurm $PROFILER)
    success "SUBMITTED -> $JOB_ID"
fi


